﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2011/03/09 初版
//
// QR code Sample Program
// -----------------------------------------------------------------
namespace QRcode_Sample
{
    public partial class Form_QRCODESAMPLE : Form
    {
        //画像メモリ格納領域
        private FVIL.Data.CFviImage m_image = new FVIL.Data.CFviImage();
        //ビデオ入力クラス
        private FVIL.Video.CFviVideo m_video = null;
        //QRコード認識クラス
        private FVIL.QR.CFviQR m_QR = new FVIL.QR.CFviQR();
        //処理時間計測クラス
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();
        //表示オーバーレイクラス宣言
        private FVIL.GDI.CFviOverlay m_overlay = new FVIL.GDI.CFviOverlay();

        //iniファイル設定画面
        Form_BROWSEIN formbroseini = new Form_BROWSEIN();

        //iniファイル名（フルパス）
        public string m_strIniFilePath;
        //ビデオデバイスのオープン状態
        private bool m_flgVideoOpened;
        
        public Form_QRCODESAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            //使用許諾の確認
            dialogLicence licence_dialog = new dialogLicence();
            if ((licence_dialog.ShowDialog(this) != DialogResult.OK))
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

             //iniファイル選択
            fnSetIniFile();

            //描画imageのセット
            cFviImageView1.Image = m_image;
            //処理対象画像の設定
            m_QR.SrcImages[0] = m_image;
            //オーバーレイの追加
            cFviImageView1.Display.Overlays.Add(m_overlay);

        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //状態変数の初期化
            m_flgVideoOpened = false;

            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0";

            //トリガモードコンボボックス初期化
            ComboBoxTrigger.Items.Clear();
            ComboBoxTrigger.Items.Add("ノーマル");
            ComboBoxTrigger.Items.Add("内部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ");
            ComboBoxTrigger.SelectedIndex = 0;

            //QRコード検出動作モード初期化
            ComboBoxEffortLevel.Items.Clear();
            ComboBoxEffortLevel.Items.Add("通常モード");
            ComboBoxEffortLevel.Items.Add("最大モード");
            ComboBoxEffortLevel.SelectedIndex = 1;

            //QRコード検出２値化手法初期化
            ComboBoxBinMethod.Items.Clear();
            ComboBoxBinMethod.Items.Add("Average");
            ComboBoxBinMethod.Items.Add("Fix");
            ComboBoxBinMethod.Items.Add("Discrimination");
            ComboBoxBinMethod.Items.Add("AverageOffset");
            ComboBoxBinMethod.Items.Add("Kittler");
            ComboBoxBinMethod.SelectedIndex = 0;

            //QRコード検出２値化閾値初期化
            //２値化手法の初期値がAverageなので使用できなくする。
            NumericUpDownBinThreshold.Enabled = false;

        }

        /// <summary>
        /// iniファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnSetIniFile()
        {
            //iniファイルの選択
            m_strIniFilePath = "";
            if ((formbroseini.ShowDialog(this) == DialogResult.OK))
            {
                m_strIniFilePath = formbroseini.strIniFilePath;
            }

            if ((m_strIniFilePath.Length > 0))
            {
                try
                {
                    //設定ファイルのロードとデバイスオープン
                    m_video = FVIL.Video.Function.CreateInstance(m_strIniFilePath);
                    m_flgVideoOpened = true;

                    //ステータス表示の更新
                    //トリガモードの取得
                    ComboBoxTrigger.SelectedIndex = (int)m_video.Trigger;
                    //カメラ露光時間の取得
                    if (m_video.DeviceType != FVIL.Video.DeviceType.RICE001)
                    {
                        NumericUpDownExposure.Enabled = true;
                        NumericUpDownExposure.Value = m_video.ExposureTime;
                    }
                    else
                    {
                        NumericUpDownExposure.Enabled = false;
                    }
                    ButtonGrabOne.Enabled = true;

                    //パラメータ表示の更新
                    //QRコード検出動作モードの取得
                    if (m_QR.Param.EffortLevel == FVIL.QR.EffortLevel.Normal)
                    {
                        ComboBoxEffortLevel.SelectedIndex = 0;
                    }
                    else
                    {
                        ComboBoxEffortLevel.SelectedIndex = 1;
                    }
                    //２値化分散しきい値の取得
                    NumericUpDownMinVar.Value = m_QR.Param.MinVar;
                    //２値化ウィンドウサイズの取得
                    NumericUpDownWindowSize.Value = m_QR.Param.WindowSize;
                    //２値化手法の取得
                    switch(m_QR.Param.BinMethod){
                        case FVIL.QR.BinMethod.Average:
                            ComboBoxBinMethod.SelectedIndex = (int)FVIL.QR.BinMethod.Average;
                            NumericUpDownBinThreshold.Value = m_QR.Param.BinThreshold;
                            NumericUpDownBinThreshold.Enabled = false;
                            break;
                        case FVIL.QR.BinMethod.Fix:
                            ComboBoxBinMethod.SelectedIndex = (int)FVIL.QR.BinMethod.Fix;
                            NumericUpDownBinThreshold.Maximum = 255;
                            NumericUpDownBinThreshold.Minimum = 0;
                            NumericUpDownBinThreshold.Value = m_QR.Param.BinThreshold;
                            NumericUpDownBinThreshold.Enabled = true;
                            break;
                        case FVIL.QR.BinMethod.Discrimination:
                            ComboBoxBinMethod.SelectedIndex = (int)FVIL.QR.BinMethod.Discrimination;
                            NumericUpDownBinThreshold.Value = m_QR.Param.BinThreshold;
                            NumericUpDownBinThreshold.Enabled = false;
                            break;
                        case FVIL.QR.BinMethod.AverageOffset:
                            ComboBoxBinMethod.SelectedIndex = (int)FVIL.QR.BinMethod.AverageOffset;
                            NumericUpDownBinThreshold.Maximum = 255;
                            NumericUpDownBinThreshold.Minimum = -255;
                            NumericUpDownBinThreshold.Value = m_QR.Param.BinThreshold;
                            NumericUpDownBinThreshold.Enabled = true;
                            break;
                        case FVIL.QR.BinMethod.Kittler:
                            ComboBoxBinMethod.SelectedIndex = (int)FVIL.QR.BinMethod.Kittler;
                            NumericUpDownBinThreshold.Value = m_QR.Param.BinThreshold;
                            NumericUpDownBinThreshold.Enabled = false;
                            break;
                    }
                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else if (formbroseini.UseFileFlg == true)
            {
                //ファイル使用
                ButtonGrabOne.Enabled = false;
            }
            else
            {
                //iniファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {
            double msec = 0;
            try
            {
                //オーバーレイをクリア
                m_overlay.Figures.Clear();

                m_video.ChangeImageSize(m_image);

                //デバイスタイプ
                FVIL.Video.CFviContinuousGrab contgrab;
                switch (m_video.DeviceType)
                {
                    case FVIL.Video.DeviceType.FHC3312:
                        contgrab = ((FVIL.Video.CFviVideoFHC3312)m_video).ContinuousGrab;
                        //画像サイズ調整
                        m_video.ChangeImageSize(m_image);
                        //メモリロック
                        ((FVIL.Video.CFviContinuousGrabFHC3312)contgrab).MemoryLock(m_image);
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        contgrab.GrabSync(1);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                    case FVIL.Video.DeviceType.FHC3313:
                        contgrab = ((FVIL.Video.CFviVideoFHC3313)m_video).ContinuousGrab;
                        //画像サイズ調整
                        m_video.ChangeImageSize(m_image);
                        //メモリロック
                        ((FVIL.Video.CFviContinuousGrabFHC3313)contgrab).MemoryLock(m_image);
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        contgrab.GrabSync(1);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                    default:
                        //時間測定開始
                        m_timer.Start();
                        //カメラ取り込み
                        m_video.GrabImageSync(m_image);
                        //時間測定終了
                        msec = m_timer.Stop();
                        break;
                }
                //測定時間表示
                TextBoxTime.Text = msec.ToString("0.000");
                formbroseini.UseFileFlg = false;
                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Load File ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 処理対象画像を読み込みます。
        /// </remarks>
        private void ButtonLoadFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofdlg = new OpenFileDialog();

            //オーバーレイをクリア
            m_overlay.Figures.Clear();
            //リストビューをクリア
            ListView.Items.Clear();

            //入力画像ファイルのロード
            ofdlg.FileName = null;
            ofdlg.Filter = "Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.DefaultExt = "*.bmp";
            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルのロード行う為のものです。
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.LoadImageFile(ofdlg.FileName, m_image, FVIL.PixelMode.Unpacking);

                    //再描画
                    cFviImageView1.Refresh();

                    //読み込むファイル形式が固定の場合は、以下のように記述します。
                    //例1) bmpファイルの場合
                    //FVIL.File.CFviImageFileBmp file = new FVIL.File.CFviImageFileBmp();
                    //file.Load(ofdlg.FileName, m_image);

                    //例2) jpgファイルの場合
                    //FVIL.File.CFviImageFileJpg file = new FVIL.File.CFviImageFileJpg();
                    //file.Load(ofdlg.FileName, m_image);
                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

        }

        /// <summary>
        /// 実行ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// QRコード認識処理を実行します。
        /// </remarks>
        private void ButtonExecute_Click(object sender, EventArgs e)
        {
            double msec = 0;

            //オーバーレイをクリア
            m_overlay.Figures.Clear();
            //リストビューをクリア
            ListView.Items.Clear();

            try
            {
                //処理対象画像の有効性検査
                //CheckValidity()は引数に指定された画像オブジェクトが処理可能か否かを判定します
                //画像オブジェクトが処理可能である（固定である）場合などは必要ありません。
                Int32 valid = FVIL.QR.CFviQR.CheckValidity(m_image);
                if (valid != FVIL.ErrorCode._SUCCESS)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", valid), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                //パラメータ設定
                //QRコード検出動作モードの設定
                if (ComboBoxEffortLevel.SelectedIndex == 0)
                {
                    m_QR.Param.EffortLevel = FVIL.QR.EffortLevel.Normal;
                }
                else
                {
                    m_QR.Param.EffortLevel = FVIL.QR.EffortLevel.Maximum;
                }
                //２値化分散しきい値の設定
                m_QR.Param.MinVar = (int)NumericUpDownMinVar.Value;
                //２値化ウィンドウサイズの設定
                m_QR.Param.WindowSize = (int)NumericUpDownWindowSize.Value;
                //２値化手法の設定
                switch(ComboBoxBinMethod.SelectedIndex){
                    case 0:
                        m_QR.Param.BinMethod = FVIL.QR.BinMethod.Average;
                        break;
                    case 1:
                        m_QR.Param.BinMethod = FVIL.QR.BinMethod.Fix;
                        break;
                    case 2:
                        m_QR.Param.BinMethod = FVIL.QR.BinMethod.Discrimination;
                        break;
                    case 3:
                        m_QR.Param.BinMethod = FVIL.QR.BinMethod.AverageOffset;
                        break;
                    case 4:
                        m_QR.Param.BinMethod = FVIL.QR.BinMethod.Kittler;
                        break;
                }
                //２値化閾値の設定
                m_QR.Param.BinThreshold = (int)NumericUpDownBinThreshold.Value;

                //時間測定開始
                m_timer.Start();
                //QRコード検出実行
                m_QR.Execute();
                //時間測定終了
                msec = m_timer.Stop();
                //測定時間表示
                TextBoxTime.Text = msec.ToString("0.000");

                //解析結果の描画
                FVIL.QR.CFviQRData data = new FVIL.QR.CFviQRData();
                data.Handle = m_QR.Result.Handle;

                //取得した塊の情報をオーバーレイへ描画
                for (Int32 i = 0; i < m_QR.Result.DecodedIndexes.Count; i++)
                {
                    data.Index = m_QR.Result.DecodedIndexes[i];

                    //描画用: 複合図形
                    FVIL.GDI.CFviGdiCluster ol_cluster = new FVIL.GDI.CFviGdiCluster();

                    //中心
                    FVIL.GDI.CFviGdiPoint ol_center = new FVIL.GDI.CFviGdiPoint();                  //描画用点データ
                    ol_center.CopyFrom(data.Center);                                                //インスタンスの複製
                    ol_center.Pen.Color = Color.FromArgb(0xFF, 0, 0);                               //描画色設定
                    ol_center.Style = FVIL.GDI.FigureStyle.Cross;                                   //各点の形状
                    ol_center.Size = new System.Drawing.Size(5, 3);                                 //描画サイズ設定

                    //ファインダーパタン、アライメントパタンの位置の取得
                    FVIL.GDI.CFviGdiPoints ol_appts = new FVIL.GDI.CFviGdiPoints();                 //描画用点データ
                    ol_appts.ConvertFrom(data.GetApptsAsDPNT_T_ARRAY());                            //型変換
                    ol_appts.Pen.Color = Color.FromArgb(0, 0xFF, 0xFF);                             //描画色設定
                    ol_appts.Style = FVIL.GDI.FigureStyle.Round;                                    //各点の形状
                    ol_appts.Size = new FVIL.Data.CFviPoint(3.0, 3.0);                              //描画サイズ設定

                    //セル位置の取得
                    //本サンプルでは表示しない。
                    //FVIL.GDI.CFviGdiPoints ol_cellOnes = new FVIL.GDI.CFviGdiPoints();            //描画用点データ
                    //ol_cellOnes.ConvertFrom(data.GetCellOnesAsDPNT_T_ARRAY());                    //型変換
                    //ol_cellOnes.Pen.Color = Color.FromArgb(0xFF, 0, 0xFF);                        //描画色設定
                    //ol_cellOnes.Style = FVIL.GDI.FigureStyle.Square;                              //各点の形状
                    //ol_cellOnes.Size = new FVIL.Data.CFviPoint(0.5, 0.5);                         //描画サイズ設定

                    //区切り位置の取得
                    FVIL.GDI.CFviGdiPolyline ol_DelimitingPts = new FVIL.GDI.CFviGdiPolyline();     //多角形描画用データ
                    ol_DelimitingPts.CopyFrom(data.GetDelimitingPtsAsCFviPolyline());               //インスタンスの複製
                    ol_DelimitingPts.Pen.Color = Color.FromArgb(0, 0, 0xFF);                        //描画色設定
                    ol_DelimitingPts.FillAlpha = 0x3F;                                              //塗り潰しの透明度
                    ol_DelimitingPts.FillColor = Color.Blue;                                        //塗り潰し色(0x0000FF)
                    ol_DelimitingPts.FillEnable = true;                                             //可視属性

                    //複合図形
                    ol_cluster.Figures.Clear();
                    ol_cluster.Figures.Add(ol_center);
                    ol_cluster.Figures.Add(ol_appts);
                    //ol_cluster.Figures.Add(ol_cellOnes);
                    ol_cluster.Figures.Add(ol_DelimitingPts);

                    //複合図形の基準位置を設定する
                    ol_cluster.BasePosition = ol_center.ToCFviPoint();

                    //オーバーレイへ追加
                    m_overlay.Figures.Add(ol_cluster);
                    m_overlay.Enable = true;
                    m_overlay.Scaling = true;
                    m_overlay.ScalingMode = FVIL.GDI.ScalingMode.Center;

                    //リストビューに追加
                    ListViewItem item = new ListViewItem(i.ToString());
                    ListView.Items.Add(item);
                    item.SubItems.Add(new ListViewItem.ListViewSubItem(item, data.Text.ToString()));
                    item.SubItems.Add(new ListViewItem.ListViewSubItem(item, data.Center.X.ToString("0.00")));
                    item.SubItems.Add(new ListViewItem.ListViewSubItem(item, data.Center.Y.ToString("0.00")));
                }
                //オーバーレイを描画
                m_overlay.Enable = true;
                //ビューの更新
                cFviImageView1.Refresh();
                }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// 処理範囲設定ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像処理の領域を設定します。
        /// </remarks>
        private void ButtonProcessSetArea_Click(object sender, EventArgs e)
        {
            if (cFviImageView1.GetOverlayActive(FVIL.GDI.CFviDrawProcarea.OverlayID))
            {
                cFviImageView1.SetOverlayActive(FVIL.GDI.CFviDrawProcarea.OverlayID, false);
            }
            else
            {
                //処理範囲選択モードに変更
                cFviImageView1.SetOverlayActive(FVIL.GDI.CFviDrawProcarea.OverlayID, true);
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog ofdlg = new SaveFileDialog();
            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);

                    //保存するファイル形式が固定の場合は、以下のように記述します。
                    //例1) bmpファイルの場合
                    //FVIL.File.CFviImageFileBmp file = new FVIL.File.CFviImageFileBmp();
                    //file.Save(ofdlg.FileName, m_image);

                    //例2) jpgファイルの場合
                    //FVIL.File.CFviImageFileJpg file = new FVIL.File.CFviImageFileJpg();
                    //file.Save(ofdlg.FileName, m_image);
                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch (ex.ErrorCode)
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

        }

        /// <summary>
        /// リストのクリックに応じて表示を変更
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// リストのアイテムへのクリックに応じてオーバレイの表示を変更。
        /// </remarks>
        private void ListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            Int32 index = 0;

            //選択したリストのみオーバーレイを表示する、リスト以外を選択した場合全オーバーレイを表示する。
            for(Int32 i = 0; i < ListView.Items.Count; i++)
            {
                ListViewItem item = ListView.Items[i];
                Boolean bEnable;

                //リスト内を選択したか判定
                if (ListView.SelectedItems.Count > 0)
                {
                    bEnable = (item.Checked || item.Selected);      //選択したアイテムの場合TRUEを設定し、それ以外はFALSEを設定する
                }
                else
                {
                    bEnable = true;                                 //リスト以外を選択した場合全てTRUEを設定する
                }

                m_overlay.Figures[index].Enable = bEnable;          //選択したアイテムを表示（未表示）にする
                index = index + 1;
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxTrigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.Trigger = (uint)ComboBoxTrigger.SelectedIndex;
                    //弊社キャプチャボードはトリガーモードの設定後にセットアップタイムが必要になります｡
                    //本プログラムのセットアップタイムは必要最大値である350msを設定しています。
                    //キャプチャボードによってこの時間は異なります。
                    System.Threading.Thread.Sleep(350);
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_flgVideoOpened == true)
                {
                    m_video.ExposureTime = (uint)NumericUpDownExposure.Value;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// ２値化手法コンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ２値化手法の設定。
        /// </remarks>
        private void ComboBoxBinMethod_SelectionChangeCommitted(object sender, EventArgs e)
        {
            switch (ComboBoxBinMethod.SelectedIndex)
            {
                case 0:
                    m_QR.Param.BinMethod = FVIL.QR.BinMethod.Average;
                    NumericUpDownBinThreshold.Enabled = false;
                    break;
                case 1:
                    m_QR.Param.BinMethod = FVIL.QR.BinMethod.Fix;
                    NumericUpDownBinThreshold.Maximum = 255;
                    NumericUpDownBinThreshold.Minimum = 0;
                    NumericUpDownBinThreshold.Value = m_QR.Param.BinThreshold;
                    NumericUpDownBinThreshold.Enabled = true;
                    break;
                case 2:
                    m_QR.Param.BinMethod = FVIL.QR.BinMethod.Discrimination;
                    NumericUpDownBinThreshold.Enabled = false;
                    break;
                case 3:
                    m_QR.Param.BinMethod = FVIL.QR.BinMethod.AverageOffset;
                    NumericUpDownBinThreshold.Maximum = 255;
                    NumericUpDownBinThreshold.Minimum = -255;
                    NumericUpDownBinThreshold.Value = m_QR.Param.BinThreshold;
                    NumericUpDownBinThreshold.Enabled = true;
                    break;
                case 4:
                    m_QR.Param.BinMethod = FVIL.QR.BinMethod.Kittler;
                    NumericUpDownBinThreshold.Enabled = false;
                    break;
            }
        }

        /// <summary>
        /// x1ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります

            //表示倍率の設定。
            cFviImageView1.Display.Magnification = 1.0;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            cFviImageView1.FitImageSize();
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if ((Value <= 0))
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Enable = true;
                cFviImageView1.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Enable = false;
                cFviImageView1.Display.OverlayAscope.Active = false;
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FVC051CHSAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            // ビデオデバイスがオープンされていた場合はクローズする
            if (m_flgVideoOpened) 
            {
                m_video.Close();
                m_flgVideoOpened = false;
            }
        }

    }
}